/*
 * quaternion.cpp
 *
 *  Created on: Jun 17, 2024
 *      Author: Dario
 */

#include <sta/math/quaternion.hpp>

#include <math.h>

namespace sta
{
	namespace math
	{
		Quaternion::Quaternion(float w, float x, float y, float z)
	        : x{x}, y{y}, z{z}, w{w}
	    {

	    }

	    Quaternion::Quaternion()
	    	: x{0}, y{0}, z{0}, w{1}
	    {

	    }

	    Quaternion Quaternion::integrate(float dt, float ox, float oy, float oz)
	    {
	    	ox *= (M_PI / 180.0f);
			oy *= (M_PI / 180.0f);
			oz *= (M_PI / 180.0f);

			float norm = sqrt(fmax(ox*ox + oy*oy + oz*oz, 0.000001));
			float dt2 = dt / 2;

			float cosDt2 = cos(norm * dt2);
			float sinDt2 = 1/norm * sin(norm * dt2);

			float qw = cosDt2*w 	+ -sinDt2*ox*x 	+ -sinDt2*oy*y 	+ -sinDt2*oz*z;
			float qx = sinDt2*ox*w 	+ cosDt2*x 		+ sinDt2*oz*y 	+ -sinDt2*oy*z;
			float qy = sinDt2*oy*w 	+ -sinDt2*oz*x 	+ cosDt2*y 		+ sinDt2*ox*z;
			float qz = sinDt2*oz*w 	+  sinDt2*oy*x 	+ -sinDt2*ox*y 	+ cosDt2*z;

			return Quaternion(qw, qx, qy, qz);
	    }

	    Quaternion Quaternion::unit()
	    {
	    	return Quaternion();
	    }

	    float Quaternion::norm()
	    {
	        return sqrtf(x*x + y*y + z*z + w*w);
	    }

	    Quaternion Quaternion::normalized()
	    {
	        float n = norm();

	        return Quaternion(w / n, x / n, y / n, z / n);
	    }

	    Quaternion Quaternion::operator+(const Quaternion& quat)
	    {
	        return Quaternion(w + quat.w, x + quat.x, y + quat.y, z + quat.z);
	    }

	    Quaternion Quaternion::operator*(float scalar)
	    {
	        return Quaternion(w * scalar, x * scalar, y * scalar, z * scalar);
	    }
	}  // namespace math
} // namespace sta


